from math import *

# Compare floating point numbers x and y
# Returns 1 if x > y
# Returns -1 if x < y
# Returns 0 if x==y to within epsilon
def compare(x,y,epsilon):
  if abs(x-y)<epsilon: return 0
  if x < y: return -1
  return 1

# Cross product of two 3D vectors
def crossprod(x,y):
  res = [x[1]*y[2] - x[2]*y[1], x[2]*y[0] - x[0]*y[2], x[0]*y[1] - x[1]*y[0]]
  norm = sqrt(res[0]*res[0] + res[1]*res[1] + res[2]*res[2])
  res = [res[0]/norm, res[1]/norm, res[2]/norm]
  return res

# Dot product of two 3D vectors
def dotprod(x,y):
  res = x[0]*y[0] + x[1]*y[1] + x[2]*y[2]
  return res

# Distance between two points in 3D
def findDist(x,y):
  diff = [x[0]-y[0],x[1]-y[1],x[2]-y[2]]
  return sqrt(dotprod(diff, diff))

# Magnitude of vector
def findNorm(x):
  return sqrt(x[0]*x[0] + x[1]*x[1] + x[2]*x[2])

# An a correction for atan2?
def atan22(y,x):
  if x>0: return atan(y/x)
  if x<0: return atan(y/x)
  return atan2(y,x)

# Finds the point f intersection (or closest approach) of two lines in 3D
def findIntersection(d1,p1,d2,p2):
  v1 = normalize(d1)
  v2 = normalize(d2)
  v12 = dotprod(v1,v2)
  xx = v12*dotprod([v2[0],v2[1],v2[2]],[p1[0]-p2[0],p1[1]-p2[1],p1[2]-p2[2]])
  xx = xx + dotprod([v1[0],v1[1],v1[2]],[p2[0]-p1[0],p2[1]-p1[1],p2[2]-p1[2]])
  fact = xx/(1.0-v12**2)
  return [p1[0]+fact*v1[0],p1[1]+fact*v1[1],p1[2]+fact*v1[2]]

# Returns a 3D vector normalized to unit length
def normalize(x):
  norm =sqrt(x[0]*x[0] + x[1]*x[1] + x[2]*x[2])
  res = [x[0]/norm, x[1]/norm, x[2]/norm]
  return res

def shaveFace(fo,v,f,no,offset):
  nf = len(f[no])
  if nf<3 or nf>4:
    print("shaveFace: Wrong number of faces ",nf,".")
    return

  v10 = [v[f[no][1]][0]-v[f[no][0]][0],v[f[no][1]][1]-v[f[no][0]][1],v[f[no][1]][2]-v[f[no][0]][2]]
  v21 = [v[f[no][2]][0]-v[f[no][1]][0],v[f[no][2]][1]-v[f[no][1]][1],v[f[no][2]][2]-v[f[no][1]][2]]
  v02 = [v[f[no][0]][0]-v[f[no][2]][0],v[f[no][0]][1]-v[f[no][2]][1],v[f[no][0]][2]-v[f[no][2]][2]]
  if nf==4:
    v32 = [v[f[no][3]][0]-v[f[no][2]][0],v[f[no][3]][1]-v[f[no][2]][1],v[f[no][3]][2]-v[f[no][2]][2]]
    v03 = [v[f[no][0]][0]-v[f[no][3]][0],v[f[no][0]][1]-v[f[no][3]][1],v[f[no][0]][2]-v[f[no][3]][2]]

  n1 = crossprod(v10,v02)
  eoffset = offset
 
  mid10 = [(v[f[no][1]][0]+v[f[no][0]][0])/2,(v[f[no][1]][1]+v[f[no][0]][1])/2,(v[f[no][1]][2]+v[f[no][0]][2])/2]
  mid21 = [(v[f[no][2]][0]+v[f[no][1]][0])/2,(v[f[no][2]][1]+v[f[no][1]][1])/2,(v[f[no][2]][2]+v[f[no][1]][2])/2]
  mid02 = [(v[f[no][0]][0]+v[f[no][2]][0])/2,(v[f[no][0]][1]+v[f[no][2]][1])/2,(v[f[no][0]][2]+v[f[no][2]][2])/2]
  if nf==4:
    mid32 = [(v[f[no][3]][0]+v[f[no][2]][0])/2,(v[f[no][3]][1]+v[f[no][2]][1])/2,(v[f[no][3]][2]+v[f[no][2]][2])/2]
    mid03 = [(v[f[no][0]][0]+v[f[no][3]][0])/2,(v[f[no][0]][1]+v[f[no][3]][1])/2,(v[f[no][0]][2]+v[f[no][3]][2])/2]
  out10 = crossprod(n1,v10)
  out21 = crossprod(n1,v21)
  out02 = crossprod(n1,v02)
  if nf==4:
    out32 = crossprod(n1,v32)
    out03 = crossprod(n1,v03)
  p10 = [0.0,0.0,0.0]
  p21 = [0.0,0.0,0.0]
  p02 = [0.0,0.0,0.0]
  if nf==4:
    p32 = [0.0,0.0,0.0]
    p03 = [0.0,0.0,0.0]

  for j in range(0,3): # This makes the face bigger than the original
    p10[j] = mid10[j] + eoffset*out10[j]
    p21[j] = mid21[j] + eoffset*out21[j]
    p02[j] = mid02[j] + eoffset*out02[j]
    if nf==4:
      p32[j] = mid32[j] + eoffset*out32[j]
      p03[j] = mid03[j] + eoffset*out03[j]
  newverts = []
  newverts.append(findIntersection(v10,p10,v02,p02))
  newverts.append(findIntersection(v10,p10,v21,p21))
  if nf==3:
    newverts.append(findIntersection(v02,p02,v21,p21))
  if nf==4:
    newverts.append(findIntersection(v32,p32,v21,p21))
    newverts.append(findIntersection(v03,p03,v32,p32))
#  newverts.append(v[f[no][0]]) # Use this for the original vertices
#  newverts.append(v[f[no][1]])
#  newverts.append(v[f[no][2]])

#  print "Original vertex 0: ",v[f[no][0]][0],v[f[no][0]][1],v[f[no][0]][2]
#  print "New vertex 0: ",newverts[0][0],newverts[0][1],newverts[0][2]

  fo.write('  polyhedron(points=[')
  for i in range(0,nf):
    fo.write('[')
    for j in range(0,3):
      output = '%12f' % (newverts[i][j]-offset*n1[j])
      fo.write(output)
      if (j<2): fo.write(',')
    fo.write(']')
    fo.write(',\n    ')
  for i in range(0,nf):
    fo.write('[')
    for j in range(0,3):
      output = '%12f' % (newverts[i][j]+offset*n1[j])
      fo.write(output)
      if (j<2): fo.write(',')
    fo.write(']')
    if (i<nf-1): fo.write(',\n    ')
  fo.write('],\n')
  if nf==3:
    fo.write('   faces=[[0,2,1],[3,4,5],[0,1,4],[0,4,3],[1,2,5],[1,5,4],[2,0,3],[2,3,5]]);\n')
  if nf==4:
    fo.write('   faces=[[0,3,2,1],[4,5,6,7],[0,1,5,4],[1,2,6,5],[2,3,7,6],[3,0,4,7]]);\n')
# reversed faces for visualization
#    fo.write('   faces=[[1,2,3,0],[7,6,5,4],[4,5,1,0],[5,6,2,1],[6,7,3,2],[7,4,0,3]]);\n')

# Returns the angles for the given face number which describe how to rotate a cube
# so that a cube face is parallel to the face
def getAngles(v,f,no):
  v10 = [v[f[no][1]][0]-v[f[no][0]][0],v[f[no][1]][1]-v[f[no][0]][1],v[f[no][1]][2]-v[f[no][0]][2]]
  v20 = [v[f[no][2]][0]-v[f[no][0]][0],v[f[no][2]][1]-v[f[no][0]][1],v[f[no][2]][2]-v[f[no][0]][2]]
  v21 = [v[f[no][2]][0]-v[f[no][1]][0],v[f[no][2]][1]-v[f[no][1]][1],v[f[no][2]][2]-v[f[no][1]][2]]
  n1 = crossprod(v20,v10)
#  print "Face ",no," normal is ",n1

# This works (sort of)
#  angles = [180.0/pi*atan22(n1[1],n1[0]), 180.0/pi*atan22(sqrt(n1[0]*n1[0]+n1[1]*n1[1]),n1[2])]
  angles = [180.0/pi*atan2(n1[1],n1[0]), 180.0/pi*atan2(sqrt(n1[0]*n1[0]+n1[1]*n1[1]),n1[2])]
  if abs(angles[0])<1.0e-4: angles[0] = 0.0
  if abs(angles[1])<1.0e-4: angles[1] = 0.0
  return angles

# Finds the center point of the given face number
def getFaceCenter(v,f,no):
  m0 = 0.50
  m1 = 0.25
  m2 = 0.25
  cx = m0*v[f[no][0]][0]+m1*v[f[no][1]][0]+m2*v[f[no][2]][0]
  cy = m0*v[f[no][0]][1]+m1*v[f[no][1]][1]+m2*v[f[no][2]][1]
  cz = m0*v[f[no][0]][2]+m1*v[f[no][1]][2]+m2*v[f[no][2]][2]
  centers = [cx,cy,cz]
  return centers

# Returns a normal to the given face number (not unit length)
def getFaceNormal(v,f,no):
  v10 = [v[f[no][1]][0]-v[f[no][0]][0],v[f[no][1]][1]-v[f[no][0]][1],v[f[no][1]][2]-v[f[no][0]][2]]
  v20 = [v[f[no][2]][0]-v[f[no][0]][0],v[f[no][2]][1]-v[f[no][0]][1],v[f[no][2]][2]-v[f[no][0]][2]]
  v21 = [v[f[no][2]][0]-v[f[no][1]][0],v[f[no][2]][1]-v[f[no][1]][1],v[f[no][2]][2]-v[f[no][1]][2]]
  n1 = crossprod(v20,v10)
  return n1
