# Write a polyhedron for OpenSCAD
# fo is the output stream
# v is the vertices
# f is the faces (all assumed to be triangles)
def writePolyhedron(fo,v,f):
  fo.write('  polyhedron(points=[[')
  for i in range(0,len(v)):
    if i>0: fo.write('    [')
    for j in range(0,3):
#      fo.write(str(v[i][j]))
      output = '%12f' % (v[i][j])
      fo.write(output)
      if (j<2): fo.write(',')
    fo.write(']')
    if (i<len(v)-1):
      fo.write(',')
      if ((i%2)==1): fo.write('\n    ');
  fo.write('],\n')
  fo.write('  faces=[')

  for i in range(0,len(f)):
    if len(f[i])>3 and f[i][3]<0: continue
    printed = printed+1
    fo.write('[')
    for j in range(0,len(f[i])):
      fo.write(str(f[i][j]))
      if (j<len(f[i])-1): fo.write(',')
    fo.write(']')
    if (i<len(f)-1):
      fo.write(',')
    if ((i%6)==5): fo.write('\n    ');
  fo.write(']);\n')

# Write a polyhedron for OpenSCAD
# Assuming some vertices are not used
# fo is the output stream
# v is the vertices
# f is the faces
# if code==1, will convert square faces to two triangular faces
# if verbosity==1, the vertex map is printed (as an openSCAD comment)
def writePolyhedronSpecial(fo,v,f,code=0,verbosity=0):
# Determine which vertices are used
  usedv = []
  hash = dict() # Empty hash table

  for i in range(0,len(v)):
    found = 0
    for j in range(0,len(f)):
      for k in range(0,len(f[j])):
        if f[j][k]==i: found=1
    if found>0:
      hash[i] = len(usedv)
      usedv.append(i)

  firsv = usedv[0]
  lastv = usedv[len(usedv)-1]

  if verbosity>0:
    fo.write('// Vertex map:')
    p = 0
    for i in range(0,len(v)):
      if i not in hash: continue
      p = p + 1
      fo.write(str(hash[i])+'->'+str(i))
      if (i!=len(v)-1): fo.write(', ')
      if ((p%6)==5): fo.write('\n//    ');
    fo.write('\n')

  fo.write('  polyhedron(points=[[')
  for i in range(0,len(v)):
    if i not in hash: continue
    if i>firsv: fo.write('    [')
    for j in range(0,3):
#      fo.write(str(v[i][j]))
      output = '%12f' % (v[i][j])
      fo.write(output)
      if (j<2): fo.write(',')
    fo.write(']')
    if (i!=lastv):
      fo.write(',\n')
  fo.write('],\n')

# Output 4 vertex faces
  if code!=1:
    fo.write('  faces=[')
    for i in range(0,len(f)):
      if len(f[i])>3 and f[i][3]<0: continue
      fo.write('[')
      for j in range(0,len(f[i])):
        fo.write(str(hash[f[i][j]]))
        if (j<len(f[i])-1): fo.write(',')
      fo.write(']')
      if (i<len(f)-1):
        fo.write(',')
        if ((i%6)==5): fo.write('\n    ');
    fo.write(']);\n')

# Convert 4-vertex faces to 3-vertex faces
  else:
    fo.write('  faces=[')
    for i in range(0,len(f)):
      if len(f[i])>3 and f[i][3]<0: continue
      fo.write('[')
      for j in [0,1,2]:
        fo.write(str(hash[f[i][j]]))
        if (j<len(f[i])-2): fo.write(',')
      fo.write('],')
      fo.write('[')
      for j in [0,2,3]:
        fo.write(str(hash[f[i][j]]))
        if (j<len(f[i])-1): fo.write(',')
      fo.write(']')
      if (i<len(f)-1):
        fo.write(',')
        if ((i%3)==2): fo.write('\n    ');
    fo.write(']);\n')
